@@ -35,12 +35,33 @@ def _parse_datediff(args: t.List) -> exp.Expression:
 
 
 class Spark(Spark2):
+    class Tokenizer(Spark2.Tokenizer):
+        RAW_STRINGS = [
+            (prefix + q, q)
+            for q in t.cast(t.List[str], Spark2.Tokenizer.QUOTES)
+            for prefix in ("r", "R")
+        ]
+
     class Parser(Spark2.Parser):
         FUNCTIONS = {
             **Spark2.Parser.FUNCTIONS,
+            "ANY_VALUE": lambda args: exp.AnyValue(
+                this=seq_get(args, 0), ignore_nulls=seq_get(args, 1)
+            ),
             "DATEDIFF": _parse_datediff,
         }
 
+        FUNCTION_PARSERS = Spark2.Parser.FUNCTION_PARSERS.copy()
+        FUNCTION_PARSERS.pop("ANY_VALUE")
+
+        def _parse_generated_as_identity(
+            self,
+        ) -> exp.GeneratedAsIdentityColumnConstraint | exp.ComputedColumnConstraint:
+            this = super()._parse_generated_as_identity()
+            if this.expression:
+                return self.expression(exp.ComputedColumnConstraint, this=this.expression)
+            return this
+
     class Generator(Spark2.Generator):
         TYPE_MAPPING = {
             **Spark2.Generator.TYPE_MAPPING,
@@ -55,10 +76,20 @@ class Spark(Spark2):
             exp.TimestampAdd: lambda self, e: self.func(
                 "DATEADD", e.args.get("unit") or "DAY", e.expression, e.this
             ),
+            exp.TryCast: lambda self, e: self.trycast_sql(e)
+            if e.args.get("safe")
+            else self.cast_sql(e),
         }
+        TRANSFORMS.pop(exp.AnyValue)
         TRANSFORMS.pop(exp.DateDiff)
         TRANSFORMS.pop(exp.Group)
 
+        def computedcolumnconstraint_sql(self, expression: exp.ComputedColumnConstraint) -> str:
+            return f"GENERATED ALWAYS AS ({self.sql(expression, 'this')})"
+
+        def anyvalue_sql(self, expression: exp.AnyValue) -> str:
+            return self.function_fallback_sql(expression)
+
         def datediff_sql(self, expression: exp.DateDiff) -> str:
             unit = self.sql(expression, "unit")
             end = self.sql(expression, "this")
