@@ -68,11 +68,17 @@ def eliminate_distinct_on(expression: exp.Expression) -> exp.Expression:
 
         if order:
             window.set("order", order.pop().copy())
+        else:
+            window.set("order", exp.Order(expressions=[c.copy() for c in distinct_cols]))
 
         window = exp.alias_(window, row_number)
         expression.select(window, copy=False)
 
-        return exp.select(*outer_selects).from_(expression.subquery()).where(f'"{row_number}" = 1')
+        return (
+            exp.select(*outer_selects)
+            .from_(expression.subquery("_t"))
+            .where(exp.column(row_number).eq(1))
+        )
 
     return expression
 
@@ -126,7 +132,7 @@ def remove_precision_parameterized_types(expression: exp.Expression) -> exp.Expr
     """
     for node in expression.find_all(exp.DataType):
         node.set(
-            "expressions", [e for e in node.expressions if not isinstance(e, exp.DataTypeSize)]
+            "expressions", [e for e in node.expressions if not isinstance(e, exp.DataTypeParam)]
         )
 
     return expression
@@ -140,7 +146,7 @@ def unnest_to_explode(expression: exp.Expression) -> exp.Expression:
 
             if isinstance(unnest, exp.Unnest):
                 alias = unnest.args.get("alias")
-                udtf = exp.Posexplode if unnest.args.get("ordinality") else exp.Explode
+                udtf = exp.Posexplode if unnest.args.get("offset") else exp.Explode
 
                 expression.args["joins"].remove(join)
 
@@ -157,63 +163,149 @@ def unnest_to_explode(expression: exp.Expression) -> exp.Expression:
     return expression
 
 
-def explode_to_unnest(expression: exp.Expression) -> exp.Expression:
-    """Convert explode/posexplode into unnest (used in hive -> presto)."""
-    if isinstance(expression, exp.Select):
-        from sqlglot.optimizer.scope import Scope
+def explode_to_unnest(index_offset: int = 0) -> t.Callable[[exp.Expression], exp.Expression]:
+    def _explode_to_unnest(expression: exp.Expression) -> exp.Expression:
+        """Convert explode/posexplode into unnest (used in hive -> presto)."""
+        if isinstance(expression, exp.Select):
+            from sqlglot.optimizer.scope import Scope
 
-        taken_select_names = set(expression.named_selects)
-        taken_source_names = {name for name, _ in Scope(expression).references}
+            taken_select_names = set(expression.named_selects)
+            taken_source_names = {name for name, _ in Scope(expression).references}
 
-        for select in expression.selects:
-            to_replace = select
+            def new_name(names: t.Set[str], name: str) -> str:
+                name = find_new_name(names, name)
+                names.add(name)
+                return name
 
-            pos_alias = ""
-            explode_alias = ""
+            arrays: t.List[exp.Condition] = []
+            series_alias = new_name(taken_select_names, "pos")
+            series = exp.alias_(
+                exp.Unnest(
+                    expressions=[exp.GenerateSeries(start=exp.Literal.number(index_offset))]
+                ),
+                new_name(taken_source_names, "_u"),
+                table=[series_alias],
+            )
 
-            if isinstance(select, exp.Alias):
-                explode_alias = select.alias
-                select = select.this
-            elif isinstance(select, exp.Aliases):
-                pos_alias = select.aliases[0].name
-                explode_alias = select.aliases[1].name
-                select = select.this
+            # we use list here because expression.selects is mutated inside the loop
+            for select in expression.selects.copy():
+                explode = select.find(exp.Explode)
 
-            if isinstance(select, (exp.Explode, exp.Posexplode)):
-                is_posexplode = isinstance(select, exp.Posexplode)
+                if explode:
+                    pos_alias = ""
+                    explode_alias = ""
 
-                explode_arg = select.this
-                unnest = exp.Unnest(expressions=[explode_arg.copy()], ordinality=is_posexplode)
+                    if isinstance(select, exp.Alias):
+                        explode_alias = select.alias
+                        alias = select
+                    elif isinstance(select, exp.Aliases):
+                        pos_alias = select.aliases[0].name
+                        explode_alias = select.aliases[1].name
+                        alias = select.replace(exp.alias_(select.this, "", copy=False))
+                    else:
+                        alias = select.replace(exp.alias_(select, ""))
+                        explode = alias.find(exp.Explode)
+                        assert explode
 
-                # This ensures that we won't use [POS]EXPLODE's argument as a new selection
-                if isinstance(explode_arg, exp.Column):
-                    taken_select_names.add(explode_arg.output_name)
+                    is_posexplode = isinstance(explode, exp.Posexplode)
+                    explode_arg = explode.this
 
-                unnest_source_alias = find_new_name(taken_source_names, "_u")
-                taken_source_names.add(unnest_source_alias)
+                    # This ensures that we won't use [POS]EXPLODE's argument as a new selection
+                    if isinstance(explode_arg, exp.Column):
+                        taken_select_names.add(explode_arg.output_name)
 
-                if not explode_alias:
-                    explode_alias = find_new_name(taken_select_names, "col")
-                    taken_select_names.add(explode_alias)
+                    unnest_source_alias = new_name(taken_source_names, "_u")
+
+                    if not explode_alias:
+                        explode_alias = new_name(taken_select_names, "col")
+
+                        if is_posexplode:
+                            pos_alias = new_name(taken_select_names, "pos")
+
+                    if not pos_alias:
+                        pos_alias = new_name(taken_select_names, "pos")
+
+                    alias.set("alias", exp.to_identifier(explode_alias))
+
+                    column = exp.If(
+                        this=exp.column(series_alias).eq(exp.column(pos_alias)),
+                        true=exp.column(explode_alias),
+                    )
+
+                    explode.replace(column)
 
                     if is_posexplode:
-                        pos_alias = find_new_name(taken_select_names, "pos")
-                        taken_select_names.add(pos_alias)
+                        expressions = expression.expressions
+                        expressions.insert(
+                            expressions.index(alias) + 1,
+                            exp.If(
+                                this=exp.column(series_alias).eq(exp.column(pos_alias)),
+                                true=exp.column(pos_alias),
+                            ).as_(pos_alias),
+                        )
+                        expression.set("expressions", expressions)
 
-                if is_posexplode:
-                    column_names = [explode_alias, pos_alias]
-                    to_replace.pop()
-                    expression.select(pos_alias, explode_alias, copy=False)
-                else:
-                    column_names = [explode_alias]
-                    to_replace.replace(exp.column(explode_alias))
+                    if not arrays:
+                        if expression.args.get("from"):
+                            expression.join(series, copy=False)
+                        else:
+                            expression.from_(series, copy=False)
 
-                unnest = exp.alias_(unnest, unnest_source_alias, table=column_names)
+                    size: exp.Condition = exp.ArraySize(this=explode_arg.copy())
+                    arrays.append(size)
 
-                if not expression.args.get("from"):
-                    expression.from_(unnest, copy=False)
-                else:
-                    expression.join(unnest, join_type="CROSS", copy=False)
+                    # trino doesn't support left join unnest with on conditions
+                    # if it did, this would be much simpler
+                    expression.join(
+                        exp.alias_(
+                            exp.Unnest(
+                                expressions=[explode_arg.copy()],
+                                offset=exp.to_identifier(pos_alias),
+                            ),
+                            unnest_source_alias,
+                            table=[explode_alias],
+                        ),
+                        join_type="CROSS",
+                        copy=False,
+                    )
+
+                    if index_offset != 1:
+                        size = size - 1
+
+                    expression.where(
+                        exp.column(series_alias)
+                        .eq(exp.column(pos_alias))
+                        .or_(
+                            (exp.column(series_alias) > size).and_(exp.column(pos_alias).eq(size))
+                        ),
+                        copy=False,
+                    )
+
+            if arrays:
+                end: exp.Condition = exp.Greatest(this=arrays[0], expressions=arrays[1:])
+
+                if index_offset != 1:
+                    end = end - (1 - index_offset)
+                series.expressions[0].set("end", end)
+
+        return expression
+
+    return _explode_to_unnest
+
+
+PERCENTILES = (exp.PercentileCont, exp.PercentileDisc)
+
+
+def add_within_group_for_percentiles(expression: exp.Expression) -> exp.Expression:
+    if (
+        isinstance(expression, PERCENTILES)
+        and not isinstance(expression.parent, exp.WithinGroup)
+        and expression.expression
+    ):
+        column = expression.this.pop()
+        expression.set("this", expression.expression.pop())
+        order = exp.Order(expressions=[exp.Ordered(this=column)])
+        expression = exp.WithinGroup(this=expression, expression=order)
 
     return expression
 
@@ -221,7 +313,7 @@ def explode_to_unnest(expression: exp.Expression) -> exp.Expression:
 def remove_within_group_for_percentiles(expression: exp.Expression) -> exp.Expression:
     if (
         isinstance(expression, exp.WithinGroup)
-        and isinstance(expression.this, (exp.PercentileCont, exp.PercentileDisc))
+        and isinstance(expression.this, PERCENTILES)
         and isinstance(expression.expression, exp.Order)
     ):
         quantile = expression.this.this
@@ -260,6 +352,31 @@ def epoch_cast_to_ts(expression: exp.Expression) -> exp.Expression:
     return expression
 
 
+def timestamp_to_cast(expression: exp.Expression) -> exp.Expression:
+    if isinstance(expression, exp.Timestamp) and not expression.expression:
+        return exp.cast(
+            expression.this,
+            to=exp.DataType.Type.TIMESTAMP,
+        )
+    return expression
+
+
+def eliminate_semi_and_anti_joins(expression: exp.Expression) -> exp.Expression:
+    if isinstance(expression, exp.Select):
+        for join in expression.args.get("joins") or []:
+            on = join.args.get("on")
+            if on and join.kind in ("SEMI", "ANTI"):
+                subquery = exp.select("1").from_(join.this).where(on)
+                exists = exp.Exists(this=subquery)
+                if join.kind == "ANTI":
+                    exists = exists.not_(copy=False)
+
+                join.pop()
+                expression.where(exists, copy=False)
+
+    return expression
+
+
 def preprocess(
     transforms: t.List[t.Callable[[exp.Expression], exp.Expression]],
 ) -> t.Callable[[Generator, exp.Expression], str]:
@@ -288,10 +405,13 @@ def preprocess(
 
         transforms_handler = self.TRANSFORMS.get(type(expression))
         if transforms_handler:
-            # Ensures we don't enter an infinite loop. This can happen when the original expression
-            # has the same type as the final expression and there's no _sql method available for it,
-            # because then it'd re-enter _to_sql.
             if expression_type is type(expression):
+                if isinstance(expression, exp.Func):
+                    return self.function_fallback_sql(expression)
+
+                # Ensures we don't enter an infinite loop. This can happen when the original expression
+                # has the same type as the final expression and there's no _sql method available for it,
+                # because then it'd re-enter _to_sql.
                 raise ValueError(
                     f"Expression type {expression.__class__.__name__} requires a _sql method in order to be transformed."
                 )
