@@ -216,6 +216,9 @@ class TestParser(unittest.TestCase):
         with self.assertRaises(ParseError):
             parse_one("IF(a > 0)")
 
+        with self.assertRaises(ParseError):
+            parse_one("SELECT CASE FROM x")
+
         with self.assertRaises(ParseError):
             parse_one("WITH cte AS (SELECT * FROM x)")
 
@@ -430,12 +433,12 @@ class TestParser(unittest.TestCase):
         self.assertEqual(parse_one("TIMESTAMP(1) WITH TIME ZONE").sql(), "TIMESTAMPTZ(1)")
         self.assertEqual(parse_one("TIMESTAMP(1) WITH LOCAL TIME ZONE").sql(), "TIMESTAMPLTZ(1)")
         self.assertEqual(parse_one("TIMESTAMP(1) WITHOUT TIME ZONE").sql(), "TIMESTAMP(1)")
-        self.assertEqual(parse_one("""JSON '{"x":"y"}'""").sql(), """CAST('{"x":"y"}' AS JSON)""")
+        self.assertEqual(parse_one("""JSON '{"x":"y"}'""").sql(), """PARSE_JSON('{"x":"y"}')""")
         self.assertIsInstance(parse_one("TIMESTAMP(1)"), exp.Func)
         self.assertIsInstance(parse_one("TIMESTAMP('2022-01-01')"), exp.Func)
         self.assertIsInstance(parse_one("TIMESTAMP()"), exp.Func)
         self.assertIsInstance(parse_one("map.x"), exp.Column)
-        self.assertIsInstance(parse_one("CAST(x AS CHAR(5))").to.expressions[0], exp.DataTypeSize)
+        self.assertIsInstance(parse_one("CAST(x AS CHAR(5))").to.expressions[0], exp.DataTypeParam)
         self.assertEqual(parse_one("1::int64", dialect="bigquery"), parse_one("CAST(1 AS BIGINT)"))
 
     def test_set_expression(self):
@@ -687,6 +690,31 @@ class TestParser(unittest.TestCase):
             LEFT JOIN b ON a.id = b.id
             """
         )
+
+        self.assertIsNotNone(query)
+
+        query = parse_one(
+            """
+            SELECT *
+            FROM a
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            LEFT JOIN UNNEST(ARRAY[])
+            """
+        )
+
         self.assertIsNotNone(query)
         self.assertLessEqual(time.time() - now, 0.2)
 
@@ -708,3 +736,26 @@ class TestParser(unittest.TestCase):
             parse_one("SELECT a, b ?? c ?? 'No Data' FROM z").sql(),
             "SELECT a, COALESCE(COALESCE(b, c), 'No Data') FROM z",
         )
+
+    def test_parse_intervals(self):
+        ast = parse_one(
+            "SELECT a FROM tbl WHERE a <= DATE '1998-12-01' - INTERVAL '71 days' GROUP BY b"
+        )
+
+        self.assertEqual(ast.find(exp.Interval).this.sql(), "'71'")
+        self.assertEqual(ast.find(exp.Interval).unit.assert_is(exp.Var).sql(), "days")
+
+    def test_parse_concat_ws(self):
+        ast = parse_one("CONCAT_WS(' ', 'John', 'Doe')")
+
+        self.assertEqual(ast.sql(), "CONCAT_WS(' ', 'John', 'Doe')")
+        self.assertEqual(ast.expressions[0].sql(), "' '")
+        self.assertEqual(ast.expressions[1].sql(), "'John'")
+        self.assertEqual(ast.expressions[2].sql(), "'Doe'")
+
+        # Ensure we can parse without argument when error level is ignore
+        ast = parse(
+            "CONCAT_WS()",
+            error_level=ErrorLevel.IGNORE,
+        )
+        self.assertEqual(ast[0].sql(), "CONCAT_WS()")
